{% extends "socialaccount/base_entrance.html" %}
{% load i18n %}
{% load allauth %}
{% block head_title %}
    {% trans "Login Cancelled" %}
{% endblock head_title %}
{% block content %}
    {% element h1 %}
        {% trans "Login Cancelled" %}
    {% endelement %}
    {% url 'account_login' as login_url %}
    {% element p %}
        {% blocktrans %}You decided to cancel logging in to our site using one of your existing accounts. If this was a mistake, please proceed to <a href="{{login_url}}">sign in</a>.{% endblocktrans %}
    {% endelement %}
{% endblock content %}
