{% extends "account/base_reauthenticate.html" %}
{% load i18n %}
{% load allauth %}
{% block reauthenticate_content %}
    {% element p %}
        {% blocktranslate %}Enter an authenticator code:{% endblocktranslate %}
    {% endelement %}
    {% url 'mfa_reauthenticate' as action_url %}
    {% element form form=form method="post" action=action_url %}
        {% slot body %}
            {% csrf_token %}
            {% element fields form=form unlabeled=True %}
            {% endelement %}
            {{ redirect_field }}
        {% endslot %}
        {% slot actions %}
            {% element button type="submit" tags="primary,mfa,login" %}
                {% trans "Confirm" %}
            {% endelement %}
        {% endslot %}
    {% endelement %}
{% endblock %}
