# Generated by Django 4.2.11 on 2024-05-09 06:09

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("account", "0008_emailaddress_unique_primary_email_fixup"),
    ]

    operations = [
        migrations.AddConstraint(
            model_name="emailaddress",
            constraint=models.UniqueConstraint(
                condition=models.Q(("primary", True)),
                fields=("user", "primary"),
                name="unique_primary_email",
            ),
        ),
    ]
