{% load i18n static %}
<noscript>{% translate "This functionality requires JavaScript." %}"</noscript>
<script src="{% static 'mfa/js/webauthn-json.js' %}"></script>
<script src="{% static 'mfa/js/webauthn.js' %}"></script>
<script src="{% static 'account/js/onload.js' %}"></script>
