{% extends "account/base_entrance.html" %}
{% load i18n %}
{% load allauth %}
{% load account %}
{% block head_title %}
    {% trans "Password Reset" %}
{% endblock head_title %}
{% block content %}
    {% element h1 %}
        {% trans "Password Reset" %}
    {% endelement %}
    {% if user.is_authenticated %}
        {% include "account/snippets/already_logged_in.html" %}
    {% endif %}
    {% element p %}
        {% blocktrans %}We have sent you an email. If you have not received it please check your spam folder. Otherwise contact us if you do not receive it in a few minutes.{% endblocktrans %}
    {% endelement %}
{% endblock content %}
