<!DOCTYPE html>
<html lang="en">
    <head>
        <meta name="viewport"
              content="width=device-width, initial-scale=1, shrink-to-fit=no">
        <title>allauth.headless OpenAPI Specification | django-allauth</title>
    </head>
    <body>
        <redoc spec-url="{% url 'headless:openapi_json' %}"></redoc>
        <script crossorigin="anonymous"
                integrity="sha384-70P5pmIdaQdVbxvjhrcTDv1uKcKqalZ3OHi7S2J+uzDl0PW8dO6L+pHOpm9EEjGJ"
                src="//cdn.jsdelivr.net/npm/redoc@2.5.2/bundles/redoc.standalone.js"></script>
    </body>
</html>
