{% extends "account/base_confirm_code.html" %}
{% load i18n %}
{% load allauth account %}
{% block head_title %}
    {% translate "Sign In" %}
{% endblock head_title %}
{% block title %}
    {% translate "Enter Sign-In Code" %}
{% endblock %}
{% block recipient %}
    {% if email %}
        <a href="mailto:{{ email }}">{{ email }}</a>
    {% else %}
        <a href="tel:{{ phone }}">{{ phone }}</a>
    {% endif %}
{% endblock %}
{% block action_url %}
    {% url 'account_confirm_login_code' %}
{% endblock %}
{% block extra_tags %}login{% endblock %}
