# Generated by Django 3.2.20 on 2023-10-11 09:23

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("socialaccount", "0005_socialtoken_nullable_app"),
    ]

    operations = [
        migrations.AlterField(
            model_name="socialaccount",
            name="extra_data",
            field=models.JSONField(default=dict, verbose_name="extra data"),
        ),
    ]
