{% load i18n %}
{% load allauth %}
{% load socialaccount %}
{% get_providers as socialaccount_providers %}
{% if socialaccount_providers %}
    {% if not SOCIALACCOUNT_ONLY %}
        {% element hr %}
        {% endelement %}
        {% element h2 %}
            {% translate "Or use a third-party" %}
        {% endelement %}
    {% endif %}
    {% include "socialaccount/snippets/provider_list.html" with process="login" %}
    {% include "socialaccount/snippets/login_extra.html" %}
{% endif %}
