{% extends "account/base_entrance.html" %}
{% load i18n %}
{% load allauth account %}
{% block head_title %}
    {% translate "Sign In" %}
{% endblock head_title %}
{% block content %}
    {% element h1 %}
        {% translate "Send me a sign-in code" %}
    {% endelement %}
    {% element p %}
        {% blocktranslate %}You will receive a special code for a password-free sign-in.{% endblocktranslate %}
    {% endelement %}
    {% element form form=form method="post" action=request_login_code_url tags="entrance,login" %}
        {% slot body %}
            {% csrf_token %}
            {% element fields form=form unlabeled=True %}
            {% endelement %}
            {{ redirect_field }}
        {% endslot %}
        {% slot actions %}
            {% element button type="submit" tags="prominent,login" %}
                {% translate "Request Code" %}
            {% endelement %}
        {% endslot %}
    {% endelement %}
    {% element button href=login_url tags="link" %}
        {% translate "Other sign-in options" %}
    {% endelement %}
{% endblock content %}
