{% extends "allauth/layouts/entrance.html" %}
{% load i18n %}
{% load allauth %}
{% block head_title %}
    {% translate "Account Inactive" %}
{% endblock head_title %}
{% block content %}
    {% element h1 %}
        {% translate "Account Inactive" %}
    {% endelement %}
    {% element p %}
        {% translate "This account is inactive." %}
    {% endelement %}
{% endblock content %}
