{% extends "idp/oidc/base.html" %}
{% load i18n %}
{% load allauth %}
{% block head_title %}
    {% trans "Error" %}
{% endblock head_title %}
{% block content %}
    {% element h1 %}
        {% trans "Error" %}
    {% endelement %}
    {% if error_form %}
        {{ error_form.errors }}
    {% else %}
        {% element p %}
            {{ error.description }}
        {% endelement %}
    {% endif %}
{% endblock %}
