{% extends "socialaccount/base_entrance.html" %}
{% load i18n %}
{% load allauth %}
{% block head_title %}
    {% trans "Signup" %}
{% endblock head_title %}
{% block content %}
    {% element h1 %}
        {% trans "Sign Up" %}
    {% endelement %}
    {% element p %}
        {% blocktrans with provider_name=account.get_provider.name site_name=site.name %}You are about to use your {{provider_name}} account to login to
{{site_name}}. As a final step, please complete the following form:{% endblocktrans %}
    {% endelement %}
    {% url 'socialaccount_signup' as action_url %}
    {% element form form=form method="post" action=action_url %}
        {% slot body %}
            {% csrf_token %}
            {% element fields form=form unlabeled=True %}
            {% endelement %}
            {{ redirect_field }}
        {% endslot %}
        {% slot actions %}
            {% element button type="submit" %}
                {% trans "Sign Up" %}
            {% endelement %}
        {% endslot %}
    {% endelement %}
{% endblock content %}
