{% load i18n %}
{% load account %}
{% load allauth %}
{% user_display user as user_display %}
{% element alert %}
    {% slot message %}
        <strong>{% blocktranslate %}Note{% endblocktranslate %}:</strong> {% blocktranslate %}You are already logged in as {{ user_display }}.{% endblocktranslate %}
    {% endslot %}
{% endelement %}
