{% load allauth %}
<form method="{{ attrs.method }}"
      {% if attrs.action %}action="{{ attrs.action }}"{% endif %}>
    {% slot body %}
    {% endslot %}
    {% slot actions %}
    {% endslot %}
</form>
