# Generated by Django 3.2.20 on 2023-09-03 19:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("socialaccount", "0004_app_provider_id_settings"),
    ]

    operations = [
        migrations.AlterField(
            model_name="socialtoken",
            name="app",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="socialaccount.socialapp",
            ),
        ),
    ]
