<!DOCTYPE html>
<html>
    <head>
        <link type="text/css"
              rel="stylesheet"
              href="https://cdn.jsdelivr.net/npm/swagger-ui-dist@5/swagger-ui.css">
        <title>allauth.headless OpenAPI Specification | django-allauth</title>
    </head>
    <body>
        <div id="swagger-ui"></div>
        <script crossorigin="anonymous"
                integrity="sha384-9CJCDqW5gKboEGedI4f6FbRtGJUQDktyO4ALxunoU7Zh2AaMp93UH1iVtC0SzT9/"
                src="//cdn.jsdelivr.net/npm/swagger-ui-dist@5.30.0/swagger-ui-bundle.js"></script>
        <script>
        const config = {
            url: "{% url 'headless:openapi_json' %}",
            dom_id: "#swagger-ui",
            presets: [
                SwaggerUIBundle.presets.apis,
                SwaggerUIBundle.SwaggerUIStandalonePreset
            ],
            requestInterceptor: (req) => {
                req.headers['X-CSRFToken'] = "{{csrf_token}}";
                return req;
            },
        };
        const ui = SwaggerUIBundle(config);
        </script>
    </body>
</html>
