{% extends "account/base_entrance.html" %}
{% load i18n %}
{% load allauth %}
{% block head_title %}
    {% trans "Change Password" %}
{% endblock head_title %}
{% block content %}
    {% element h1 %}
        {% if token_fail %}
            {% trans "Bad Token" %}
        {% else %}
            {% trans "Change Password" %}
        {% endif %}
    {% endelement %}
    {% if token_fail %}
        {% url 'account_reset_password' as passwd_reset_url %}
        {% element p %}
            {% blocktrans %}The password reset link was invalid, possibly because it has already been used.  Please request a <a href="{{ passwd_reset_url }}">new password reset</a>.{% endblocktrans %}
        {% endelement %}
    {% else %}
        {% element form method="post" action=action_url %}
            {% slot body %}
                {% csrf_token %}
                {{ redirect_field }}
                {% element fields form=form %}
                {% endelement %}
            {% endslot %}
            {% slot actions %}
                {% element button type="submit" name="action" %}
                    {% trans 'Change Password' %}
                {% endelement %}
                {% element button type="submit" form="logout-from-stage" tags="link,cancel" %}
                    {% translate "Cancel" %}
                {% endelement %}
            {% endslot %}
        {% endelement %}
    {% endif %}
    {% if not cancel_url %}
        <form id="logout-from-stage"
              method="post"
              action="{% url 'account_logout' %}">
            <input type="hidden" name="next" value="{% url 'account_login' %}">
            {% csrf_token %}
        </form>
    {% endif %}
{% endblock content %}
